/**
 * This example for the PIC16f628a repeats data read through serial, back
 * at a given baud.  It has been tested on a PIC16f628a operating 
 * at 4MHz with at 9600 baud.  See the PIC16f628a manual for tables of 
 * allowable clock rate and baud rate combinations.
 *
 * Copyright (C) 2007 Tyler Montbriand
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 * Free Software  Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#define __16f628a
#include "pic/pic14regs.h"
#include "tsmtypes.h"
#include "tsmpins.h"
#include "tsmpic.h"
#include "tsmuart.h"
#include "tsmtimer.h"

// Set the __CONFIG word:
Uint16 at 0x2007  __CONFIG = CONFIG_WORD;

#define BUFFER_BITS	3
#define BUFFER_MAX	(1<<BUFFER_BITS)
#define BUFFER_MASK	(BUFFER_MAX-1)

struct
{
	Uint8 bdata[1<<BUFFER_BITS];
	Uint8 rd;
	Uint8 wr;
	Uint8 full;
} buffer;

#define BUFFER_EMPTY	((buffer.rd == buffer.wr)&&(!buffer.full))

#define BUFFER_RESET()	do {	\
	buffer.rd=0;		\
	buffer.wr=0;		\
	buffer.full=0;		\
	} while(0)

Sint8 buffer_write(Sint8 c)
{
	if(buffer.full || (c<0))
		return(-1);

	buffer.bdata[buffer.wr++]=c;
	buffer.wr&=BUFFER_MASK;
	if(buffer.wr==buffer.rd)
		buffer.full=1;  
	TXIE=1;

	return(0);
}

Sint8 buffer_read(void)
{
  Sint8 tmp;
  if(BUFFER_EMPTY)
    return(-1);

  tmp=buffer.bdata[buffer.rd++];
  buffer.rd&=BUFFER_MASK;
  buffer.full=0;
  if(buffer.rd==buffer.wr)	// Out of data, stop sending
	TXIE=0;

  return(tmp);    
}

void rcif_handler(void)
{
	if(buffer_write(RCREG)<0)
	{
		PORTA=0x08;
		return;
	}

	// DEBUG:  Display write position on PORTA
	PORTA=buffer.wr;
}

void txie_handler(void)
{
	Sint8 tmp=buffer_read();
	if(tmp<0)
		return;
	TXREG=tmp;
}

void Intr(void) interrupt 0
{
	if(RCIF)	// If we've recieved a byte:
		rcif_handler();

	// We'll be turning transmit interrupts on and off depending on
	// whether we have data to transmit.
	if(TXIE && TXIF)
		txie_handler();

	if(INTF)
	{
		buffer_write('!');
		INTF=0;
	}

	if(T0IF)
	{
		if(T0CS)
			TMR0=0xff;

		buffer_write('.');
		T0IF=0;
	}

//	GIE=1;		// Globally enable interrupts.
			/**  We don't need to do this ourselves since
			 *  the compiler ALWAYS ADDS THIS FOR US
			 *  in interrupt functions!
			 *  If you try and DISable interrupts in an
			 *  interrupt function it WON'T WORK since
			 *  the compiler ALWAYS turns them back ON!
			 */

}

void send_str(const char *buf)
{
	while(*buf)
	{
		GIE=0;
		if(buffer.full)
		{
			GIE=1;
			continue;
		}

		buffer_write(*buf);
		buf++;
		GIE=1;
	}
}

enum portb0_int_edge
{
	B0_FALLING,
	B0_RISING
};

#define PORTB0_INT_INIT(EDGE)	do {	\
		INTE=1;			\
		INTEDG=EDGE;		\
	} while(0)

static void main(void)
{

#ifdef __16f628a	// Only compile this section for PIC16f628a
	CMCON = 0x07;	/** Disable comparators.  NEEDED FOR NORMAL PORTA
			 *  BEHAVIOR ON PIC16f628a!
			 */
#endif

	NOT_RBPU=0;			// Enable weak PORTB pullups

	TRISA=0xf0|TRISA_DEFAULT;	// RA0-RA3 outputs, RA4... inputs
	TRISB=0xff|TRISB_DEFAULT;	// All inputs
	PORTA=0x00;			// Clear PORTA
	PORTB=0x00;			// Clear PORTB

	INTCON=0x00;			// Disable all interrupts

	BUFFER_RESET();			// Initialize buffer before serial port!
	SERIAL_INIT(4000L, 9600L, 1);	// Configure serial for 9600N81 at 4MHz
	// Configure timer to count from RA4 rising edge
	TMR0_INIT(0, T0_EXTERNAL,T0_RISING);

	SERIAL_INT_INIT();		//Enable serial recieve interrupt
//	PORTB0_INT_INIT(B0_FALLING);	// Interrupt on B0 falling edge
	TMR0_INT_INIT();		// Enable T0 interrupts
	TMR0=0xff;			// Overflow immediately

	GIE=1;				// Globally enable interrupts

	send_str("?\r\n");

//	if(DETECT_8BIT)
//		send_str("8-bit");
//	else
//		send_str("9-bit");

	send_str("\r\n");


	while(1)
	{
		if(IS_SET(PORTA,4))
			SET_BIT(PORTA,3);
		else
			CLEAR_BIT(PORTA,3);
	}
//	{
//		while(!RCIF);	// Wait until data recieved
//		i=RCREG;	// Store for later
//		while(!TRMT);	// Wait until we're free to transmit
//		TXREG=i;	// Transmit
//	}

}
