/**
 * main.c -- main event loop, and interrupt handling.
 * (C) 2007 Tyler Montbriand
 * monttyle@burningsmell.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 *	main() handles and/or transmits packets in an infinite loop.
 *
 * 	The interrupt handler is triggered by recieved bytes,
 *	Timer 0 overflow, or a rising/falling edge on B0.
 *	Most of the actual configuration happens in the packet manager;  
 *	for instance, you might send 418000f to a stepper controller to
 *	tell it it's destination is step 15, then 4111117 to tell it
 *	to enable timer interrupts and count pulses on the SYNC line;
 *	each pulse will trigger the stepper controller to step towards 
 *	its destination.
 */
#include "pic/pic14regs.h"	// for PIC registers like PORTA
#include "reprap.h"		// For types
#include "tsmpic.h"		// Various utility macros
#include "tsmuart.h"		// Serial port configuration
#include "packet.h"		// Packet management

#ifdef CHIP_CLASS_COMM
#include "comm.h"		// comm module functions and macros
#elif defined(CHIP_CLASS_STEPPER)
#include "stepper.h"		// stepper module functions and macros
#endif

// Set the CONFIG word
Uint16 at 0x2007  __CONFIG = CONFIG_WORD;

// Incremented when a PACKET_COMPLETE notification needs to be sent to node 1
static volatile Uint8 complete=0;
// Incremented when the limit switch is hit.
static volatile Uint8 rail=0;

static void init(void);	// Set variables and registers to default values


void Intr(void) interrupt 0
{
	static Sint8 tmp;
	// While we have recieved bytes on the USART:
	while(RCIF)	// Loop while there are bytes in the hardware buffer
	{	// Attempt to append it to the current packet
		tmp=packetbuf_add_char(RCREG);
		// Display any errors on the debugging LEDS
		if(tmp) DEBUG(tmp);

		// Packet buffer filling up?  Set CTS to stop incoming data
		if(incoming.packet_used>=(NUM_PACKETS/2))
			SET_CTS();
	}

	// PORTB Pin 0 interrupt
	if(INTE && INTF)
	{
		INTF=0;		// Clear interrupt flag
#ifdef CHIP_CLASS_STEPPER
		if(INTEDG)	// Rising edge -- minimum limit triggered
		{
			// Notify main code that the operation was interrupted
			rail++;
			STEPPER_DEST=0;		// Reset position data
			stepper_pos=0;
			STEPPER_FRAC=0;
			T0IE=0;			// Disable timer interrupt
		}
#endif
	}

	// Timer 0 interrupt
	if(T0IE && T0IF)
	{
//		SEND_CHAR('.');
		T0IF=0;	// Clear interrupt flag

		// If we're triggering from an external sync, make the timer
		// register overflow on the FIRST pulse.
                if(T0CS)
                        TMR0=0xff;

#ifdef CHIP_CLASS_COMM	// comm-module-only code
		if(DRIVE_COUNT < 0)	// We have delivered n pulses
		{
			T0IE=0;		// Disable timer 0 interrupt
			complete++;	// Notify main loop
			return;
		}

		DRIVE_COUNT--;		// Need to deliver more pulses

		SET_BIT(PORTA, PIN_PORTA_T0CLK);	// Drive SYNC high
			__asm nop __endasm;	// Wait 8 clock cycles
			__asm nop __endasm;
			__asm nop __endasm;
			__asm nop __endasm;
			__asm nop __endasm;
			__asm nop __endasm;
			__asm nop __endasm;
			__asm nop __endasm;
		CLEAR_BIT(PORTA, PIN_PORTA_T0CLK);	// Drive SYNC low
#elif defined(CHIP_CLASS_STEPPER)	// stepper-module-only code
		if(stepper_move()<0)	// Done moving?
		{
			T0IE=0;		// Disable timer interrupts
			STEPPER_FRAC=0;	// Clear fractional movement
			complete++;	// Notify main loop
		}
#endif
	}
}

void main(void)
{
	static Uint8 n;

	init();			// Initialize things

	DEBUG(ERR_TRACE1);	// Turn on a debug LED to show we're cookin'

	// Send human-readable prompt
	// Not necessary, just helps debugging.
	CRITICAL(send_ee(EE_PROMPT));

	GIE=1;		// Enable interrupts

	while(1)	// Loop forever
	{
//		if(FERR | OERR)
//			DIE(ERR_HWERROR);

		if(complete)	// Operation complete?  Notify module 1.
			CRITICAL((complete--, send_ee(EE_PACKET_COMPLETE)));

		if(rail)	// Hit a limit?  Notify module 1.
			CRITICAL((rail--, send_ee(EE_PACKET_COMPLETE)));

		if(incoming.packet_used)	// Got any packets?
		{
			packetbuf_handle();	// Handle most recent
			GIE=0;			// Disable interrupts
				incoming.packet_free++;
				incoming.packet_used--;

				// Buffer running out?  enable CTS to 
				// allow incoming data
				if(incoming.packet_used<=1)
					CLEAR_CTS();
			GIE=1;	// Enable interrupts
			PACKETBUF_NEXT(incoming.rd);	// Point to next packet
		}
	}

}

static void init(void)
{
	NOT_RBPU=0;		// Enable weak PORTB pullups
	TRISA=TRISA_DEFAULT;	// Set TRISA and TRISB to default values
	TRISB=TRISB_DEFAULT;
	PORTA=0xc0;		// Why 0xc0?  I forget.  Probably unimportant.
	PORTB=0x00;		// Clear PORTB

#if defined(CHIP_CLASS_STEPPER)	// stepper-module-only code
	stepper_pos=0;		// Clear stepper position
	stepper_step=0;		// Clear stepper phase
	stepper_show();		// Show stepper bit pattern
#endif
	// Configure USART
	SPBRG=EEPROM_READ(EE_BAUD);	// Program baud rate from EEPROM values
	BRGH=EEPROM_READ(EE_BAUD_HI);

	SYNC=0;	// Asynchronous serial.
	SPEN=1;	// Enable serial port
	TXEN=1;	// Enable transmission
	CREN=1;	// Enable reception

	// Enable peripheral interrupts
	TMR0=0;

	// Clear the packet buffer
	PACKETBUF_INIT(incoming);

	CLEAR_CTS();		// Signal readiness to recieve packets
	SERIAL_INT_INIT();	// Configure serial interrupts
}
