/**
 * packet.c -- communications handling
 * Copyright (C) 2007 Tyler Montbriand
 * monttyle@burningsmell.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 *	packet.c -- the main switching yard for packet communications.  
 *	This is where packets are translated, seperated from each other, 
 *	repeated, and handled.  A stream of bytes is seperated into it's 
 *	component packets, where different actions are performed for 
 *	different packets.
 *
 *	The network architecture is a simple ring;  not a token ring, 
 *	just a ring.  The computer transmits to the first module, the 
 *	first to the second, the second to the third, etc, and the last 
 *	module transmits to the computer.  Packets are repeated from 
 *	module to module until the final destination is found.
 *
 *	Packets have a source, destination, type, and payload.  The 
 *	source is simply the chip ID of the sending module or computer,
 *	the destination is either a chip ID or a global broadcast, and
 *	the type is arbitrary.
 *
 *	When a packet is recieved, if the destination chip ID matches the 
 *	module's chip ID, the module handles the packet instead of 
 *	retransmitting it.  (Broadcast packets are handled AND 
 *	retransmitted.)
 *
 *
 *	Packets are 7 ASCII bytes followed by a terminator, and composed of
 *	the hexadecimal digits 0-9 and a-f.  The header, with source, 
 *	destination, and type information, is 3 digits; the the data payload
 *	is 4 digits, representing a 16-bit integer.
 *
 *	3100000
 *	|||\-+/
 *	|||  \--- Payload
 *	||\------ Packet Type
 *	|\------- Packet Source
 *	\-------- Packet Destination
 *
 *	Both carriage return(\r) and newline(\n) are valid packet
 *	terminators, for the sake of those typing in packets raw from
 *	a serial console.
 */
#include "pic/pic14regs.h"
#include "reprap.h"
#include "tsmpic.h"
#include "packet.h"
#include "databox.h"

// Packet buffer
volatile struct packetbuf incoming;

// Data inside our packet.  P_DATA(0) is the first nibble, etc.
#define P_DATA(X)	(incoming.buffer[(X)+incoming.rd])
// Enable interrupts, then wait for TXREG to become available
static void wait_serial_break(void);

/**
 * To conserve code space, return packets are stored in the EEPROM for 
 * 'playback' -- even for repeating packets that aren't ours.  The same
 * function can be used for most transmissions.
 *
 * Different characters subsitute different things:
 *   0x10-0x16 substitutes P_DATA(0)-P_DATA.  Invalid outside packetbuf_handle.
 *   0x17-0x1f are disallowed.
 *   # substitutes the hexadecimal value of the local chip ID.
 *   0x00 means 'stop'.
 *
 * Everything else gets transmitted as a literal character.
 *
 * The function takes the index in EEPROM memory to begin, and transmits 
 * characters one by one until it encounters a NULL character.
 */
void send_ee(Uint8 addr)
{
	Uint8 tmp;

	WAIT_READY();	// Don't transmit packets until the reciever is ready

	while(tmp=EEPROM_READ(addr++))	// Loop until NULL
	{
/*		if(tmp&0x80)
		{
			addr=tmp&(~0x80);
			tmp=EEPROM_READ(addr);
			continue;
		}*/

		if((EEDATA & 0xf0)==0x10)	// Translate to recieved bytes
			tmp=P_DATA(EEDATA&(PACKET_SIZE-1));
		else if(EEDATA == '#')		// Translate to chip ID
			tmp=EEPROM_READ(EE_CHIPID);
		else				// Literal bytes, skip tohex
			goto SKIP_CONVERSION;

		tmp=TOHEX(tmp);
SKIP_CONVERSION:
		wait_serial_break();	// Wait until TXREG ready
		TXREG=tmp;		// Send packet data
	}

	wait_serial_break();	// Wait until TXREG ready
	TXREG=PACKET_CHAR_END;	// Send packet terminator
}

// Check the most recent packet for validity then either pass it along,
// handle it ourselves, or both.
void packetbuf_handle(void)
{
	static Uint8 n,chipid;
	static Uint8 return_code;
	static Uint8 type, dest;
	static union	// Translated data payload of packet
	{
		Uint8 bytes[2];
		Sint16 word;
	} bdata;

	return_code=0;	// Default to no return packet

	GIE=0;		// Arrays are NOT INTERRUPT SAFE in PICs!

	// Convert four nibbles into one 16-bit int
        bdata.bytes[1]=(P_DATA(P_DATA0+0)<<4) | P_DATA(P_DATA0+1);
        bdata.bytes[0]=(P_DATA(P_DATA0+2)<<4) | P_DATA(P_DATA0+3);

	type=P_DATA(P_TYPE);	// Store these in local variables, because
	dest=P_DATA(P_DEST);	// array access is expensive.
	chipid=EEPROM_READ(EE_CHIPID);

	if(P_DATA(P_SOURCE) == chipid)	// Ignore packets we sent
		goto PACKET_HANDLE_RETURN;

	if(dest != chipid)		// Not our packet?  Pass along.
		send_ee(EE_PACKET_REPEAT);

	// Belongs to us, or broadcast to all?  Handle it.
	if(	(dest == chipid) ||
		(dest == PACKET_DEST_BROADCAST)	)
	switch(type)
	{
	/**	Configure timer interrupt
	 *
	 * ds1[0-1][0-1][0-1][0-7]
         *      |    |    |    \---- Prescaler, see OPTION_REG.
	 *      |    |    \--------- Rising/Falling edge, see T0SE.
         *      |    \-------------- Clock source.  See PSA, T0CS.
	 *      \------------------- Disable, Enable.  See T0IE.
	 */
	case PACKET_TIMER:
		T0IE=0;
		OPTION_REG&=~(0x3f);	// Turn off option flags
		OPTION_REG|=(bdata.bytes[0]&0x07);		

		if(bdata.bytes[0]&0x10)
			T0SE=1;

		if(bdata.bytes[1]&0x01)
		{
			T0CS=1;
			PSA=1;
			TMR0=0xff;
		}

		if(bdata.bytes[1]&0x10)
			T0IE=1;

	/* FALL THROUGH */

	/**	Ping packet
	 *	ds0xxxx
	 */
	case PACKET_PING:
		return_code=EE_PACKET_ACK;	// Return ACK as reply
		break;

	/**	Configure B0 pin interrupt
	 * ds3[0-1]xx[0-1]
	 *      |      \--- Falling Edge, Rising Edge.
	 *      \---------- Disable, Enable.
	 */
	case PACKET_INTERRUPT:
		INTE=0;		// Disable B0 interrupt
		if(bdata.bytes[0]&0x01)
		{
			INTEDG=1;	// Rising edge
			// Set flag if line alreay high
			INTF=PORTB&PORTB_INTERRUPT;
		}
		else
		{
			INTEDG=0;	// Falling edge
			// Set flag if line already low
			INTF=(PORTB&PORTB_INTERRUPT)^0x01;
		}

		if(bdata.bytes[1]&0x10)	// Enable interrupts
			INTE=1;

		return_code=EE_PACKET_ACK;
		break;

	/** Write data to data boxes 0-3
	 * ds[8-b][0-f][0-f][0-f][0-f]
	 *     |  \---+----/\---+----/
	 *     |      |         \------ Low order nibble
	 *     |      \---------------- High order nibble
         *     \----------------------- [8-b] => Data box [0-3]
	 */
	case PACKET_DATA0:
	case PACKET_DATA1:
	case PACKET_DATA2:
		DATABOX_REG(type).s16=bdata.word;	// Set data box
		return_code=EE_PACKET_ACK;		// Send ACK reply
		break;

	/**	Write data to EEPROM memory
	 * ds2[0-f][0-f][0-f][0-f]
	 *    \---+----/\---+----/
	 *        |         \------ Data byte
	 *	  \---------------- Address Byte
	 */
	case PACKET_EEPROM:
		EEPROM_WRITE(bdata.bytes[1],bdata.bytes[0]);	// Write
		return_code=EE_PACKET_ACK;			// Return ACK
		break;

	default:		// All undefined types, return NACK
		return_code=EE_PACKET_NACK;
		break;
	}

	if(return_code)	// If given a return code, send a return packet
		send_ee(return_code);

PACKET_HANDLE_RETURN:
	GIE=1;		// Re-enable interrupts
}

// Append one more char to the end of the packet or, if that character
// is a packet terminator, move to the start of the next packet.
// Returns zero on success, or a debug code for the LEDs on error.
Sint8 packetbuf_add_char(Sint8 c)
{
	Sint8 pos;

	if(incoming.packet_free == 0)	// Overflow
		return(ERR_PACKET_OVERFLOW);

	pos=incoming.wr&PACKET_MASK;	// Get index inside packet

	switch(c)
	{
	/* End-of-packet */
	case '\r':	// Terminal compatibility
	case PACKET_CHAR_END:
		if(pos!=0)	// Do nothing on empty packet
		{
			// Clear remainder of buffer
			while(incoming.wr&PACKET_MASK)
				incoming.buffer[incoming.wr++]=0;

			incoming.packet_used++;
			incoming.packet_free++;
		}

		/* FALL THROUGH -- puts buffer back in bounds */

	/* Cancel packet -- reset to starting position in packet */
	case PACKET_CHAR_CANCEL:
		incoming.wr&=BUFFER_MASK;
		return(0);

	/* Everything else presumed to be input chars */
	default:
		if(pos==PACKET_MAX)	/* Too much data for one packet */
			return(ERR_INPUT_OVERFLOW);

		// Convert ASCII to hex.  NOTE, LOWERCASE ONLY!
		c-='0';
		if(c>9)	c-=(7+32);
		if( (c<0) || (c>15) )
			return(ERR_BAD_HEX);
		break;
	}

	incoming.buffer[incoming.wr++]=c;	// Add converted data to buffer
	return(0);
}

/**
 * While we're waiting, it's safe to let interrupts happen.
 */
static void wait_serial_break(void)
{
	GIE=1;	// Enable interrupts
		WAIT_SERIAL();	// Busyloop
	GIE=0;	// Disable interrupts
}
