/**
 * packet.h -- data structures, constants, and externals for packet comms
 * Copyright (C) 2007 Tyler Montbriand
 * monttyle@burningsmell.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef __PACKET_H__
#define __PACKET_H__

#include "tsmtypes.h"

/**
 *  Offsets of bytes in the packet.  Byte 0 is the destination chipid,
 *  byte 1 is the source chipid, etc.  They're actually nibbles, but 
 *  transmitted as hexadecimal digits, so each byte contains the value 
 *  0-15.
 */
enum packet_offset_e
{
/* Bytes 0-2:	Header	*/
	P_DEST,			/* Destination chip ID	*/
	P_SOURCE,		/* Source chip ID	*/
	P_TYPE,			/* Packet type		*/
/* Bytes 3-6:	Data	*/
	P_DATA0,		/* Data nibble 0	*/
	P_DATA1,		/* Data nibble 1	*/
	P_DATA2,		/* Data nibble 2	*/
	P_DATA3,		/* Data nibble 3	*/
	P_CMD_EXT=P_DATA0	/* Extended command packets have another type. */
};

/**
 *	ASCII character values that have special meaning to the packet 
 *	manager.
 */
enum packet_char_e
{
	PACKET_CHAR_END		='\n',	/* Packet-terminating character. */
	PACKET_CHAR_CANCEL	='\x18'	/* Clears a partly-recieved packet. */
};

/**
 *	Special values of source/destination nibble
 */
enum packet_dest_e
{
	PACKET_DEST_INVALID	=0x0,	/* Destination 0 is not valid.	*/
	PACKET_DEST_HOST	=0x1,	/* 1 is the host computer	*/
	PACKET_DEST_BROADCAST	=0xf	/* Broadcast gets passed to all	*/
};

/**
 *	Defines which value of 'type' means what kind of packet
 */
enum packet_type_e
{
/*	0x0 - 0x7:	Commands	*/
	PACKET_PING		=0x0,	/* Destination chip returns ACK */
	PACKET_TIMER		=0x1,	/* Configures timer interrupt	*/
	PACKET_EEPROM		=0x2,	/* Writes an EEPROM byte	*/
	PACKET_INTERRUPT	=0x3,	/* Configure port B0 interrupt	*/
	PACKET_COMMAND4		=0x4,
	PACKET_COMMAND5		=0x5,
	PACKET_COMMAND6		=0x6,
	/* Leave room for expansion */
	PACKET_COMMAND_EXT	=0x7,	/* Unimplimented		*/
/*	0x8 - 0xb:	Data		*/
	PACKET_DATA0		=0x8,	/* Data nibble 0		*/
	PACKET_DATA1		=0x9,	/* Data nibble 1		*/
	PACKET_DATA2		=0xa,	/* Data nibble 2		*/
	/* Leave room for expansion */
	PACKET_DATA_EXT		=0xb,	/* Unimplimented		*/
/*	0xc - 0xf:	Network		*/
	PACKET_ACK		=0xc,	/* Command succeeded		*/
	PACKET_NACK		=0xd,	/* Command returned error	*/
	PACKET_COMPLETE		=0xe,	/* Operation complete		*/
	PACKET_NET_EXT		=0xf,	/* Extended packet types	*/
};

// Paramaters defining the internal structure of the packet buffer.
// Don't touch.
#define PACKET_BITS	(3)	//8-byte packets
#define PACKET_SIZE	(1<<PACKET_BITS)
#define PACKET_MAX	7
#define PACKET_MASK	(PACKET_SIZE-1)

#define BUFFER_BITS	(2)	// Room for 4 packets
#define BUFFER_SIZE	(1<<(PACKET_BITS+BUFFER_BITS))
#define BUFFER_MASK	((BUFFER_SIZE-1)^PACKET_MASK)

#define NUM_PACKETS		(1<<BUFFER_BITS)
#define NUM_PACKETS_MASK	(NUM_PACKETS-1)

struct packetbuf
{
	Uint8 rd;	// Position, in bytes, of the most recent packet, if any.
	Uint8 wr;	// Position, in bytes, of last free packet, if any.
	Uint8 packet_used;	// How many packets have been used in the buffer?
	Uint8 packet_free;	// How many packets are left free in the buffer?
	// We use it like a two-dimensional array because SDCC needs
	// to make an extra function call to use two-dimensional arrays 
	// directly.
	Uint8 buffer[BUFFER_SIZE];
};

// True if there is no room for more packets in the buffer.
#define PACKET_OVERFLOW(X)	((X).packet_free==0)

// Increments a variable and aligns it to the start of the packet boundary.
#define PACKETBUF_NEXT(X)	do {		\
	(X)+=PACKET_SIZE;			\
	(X)&=BUFFER_MASK;			\
	} while(0)

/* Initializes a packet buffer structure */
#define PACKETBUF_INIT(X)	do {		\
	(X).rd=0;				\
	(X).wr=0;				\
	(X).packet_used=0;			\
	(X).packet_free=(1<<BUFFER_BITS);	\
	} while(0)

extern volatile struct packetbuf incoming;

// Sends a packet string from EEPROM memory
extern void	send_ee(Uint8 addr);

// Handles the next packet in the buffer
extern void	packetbuf_handle(void);

// Adds a character to the buffer.  Returns 0 on success, nonzero on error.
extern Sint8	packetbuf_add_char(Sint8 c);

#endif/*__PACKET_H__*/
