/**
 * reprap.h -- physical pins, error codes, macros
 * Copyright (C) 2007 Tyler Montbriand
 * monttyle@burningsmell.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef __TSMPINS_H__
#define __TSMPINS_H__

#if SDCC < 264
#error "Your version of SDCC is too old, needs >=2.6.4"
#endif

#include "tsmpic.h"
#include "tsmee.h"

/* List of inputs on PORTA, all else is outputs */
enum porta_pins_e
{
	PIN_PORTA_T0CLK=4,
	PIN_PORTA_READY=6
};

/* List of inputs on PORTB, all else is outputs */
enum portb_pins_e
{
	/* These three are unmovable. */
	PIN_PORTB_INTERRUPT	=0,
	PIN_PORTB_RX		=1,
	PIN_PORTB_TX		=2,

	PIN_PORTB_CTS		=3
};

enum portb_mask_e
{
	PORTB_INTERRUPT	=GET_BIT(PIN_PORTB_INTERRUPT),
	PORTB_RX	=GET_BIT(PIN_PORTB_RX),
	PORTB_TX	=GET_BIT(PIN_PORTB_TX),
	PORTB_CTS	=GET_BIT(PIN_PORTB_CTS),
	PORTA_T0CLK	=GET_BIT(PIN_PORTA_T0CLK),
	PORTA_READY	=GET_BIT(PIN_PORTA_READY)
};

// What value to set TRISB to on power-on.
#define TRISB_DEFAULT		(PORTB_RX|PORTB_TX|PORTB_INTERRUPT)

// What value to set TRISA to on power-on.  Comm controllers drive the 
// sync line, other modules read it.
#ifdef CHIP_CLASS_COMM
#	define TRISA_DEFAULT		(PORTA_READY)
#else
#	define	TRISA_DEFAULT		(PORTA_T0CLK|PORTA_READY)
#endif

/**
 *	Four-bit error codes for displaying on the debug LEDs.
 */
enum err_t
{
	ERR_NONE		=0x00,	// Nobody here but us chickens
	ERR_TRACE1		=0x01,	// Reserved for debugging trace
	ERR_TRACE2		=0x02,
	ERR_TRACE3		=0x03,
	ERR_TRACE4		=0x04,
	ERR_TRACE5		=0x05,
	ERR_TRACE6		=0x06,
	ERR_TRACE7		=0x07,
	ERR_BAD_HEX		=0x08,	// Invalid character recieved
	ERR_INPUT_OVERFLOW	=0x09,	// Too many bytes for one packet
	ERR_PACKET_OVERFLOW	=0x0a,	// Too many packets
	ERR_HWERROR		=0x0f	// On fire/etc
};

// Display a 4-bit value on the debugging LEDs
#define DEBUG(X)	do { PORTB&=0x0f; PORTB|=((X)<<4); } while(0)

// Display a 4-bit value on the debugging leds, then loop forever
#define DIE(X)		do {	\
	SET_CTS();		\
	GIE=0;			\
	DEBUG(X);		\
	while(1);		\
	} while(0)

// Set the CTS pin.  This signifies that the previous sender should wait.
#define SET_CTS()	SET_BIT(PORTB, PIN_PORTB_CTS)
// Clear the CTS pin.  This signifies that the previous sender can transmit.
#define CLEAR_CTS()	CLEAR_BIT(PORTB, PIN_PORTB_CTS);

// Wait until the next reciever signals that they're ready to recieve packets.
#define WAIT_READY()	while(PORTA & PORTA_READY)

// Wait until the PIC's USART is free to be fed more bytes.
#define WAIT_SERIAL()	while(!TRMT)

// Disable interrupts, do X, enable interrupts.
#define CRITICAL(X)	do { GIE=0; (X); GIE=1; } while(0)

/**
 *	Defines offsets inside the PIC's eeprom memory.
 */
enum eeprom_off_t
{
	EE_CHIPID		=0x00,	// Chip ID, 1 byte
	EE_CHIPCLASS		=0x01,	// Chip Class, 1 byte
	EE_BAUD			=0x02,	// Baud rate register value
	EE_BAUD_HI		=0x03,	// BRGH bit value
	EE_PROMPT		=0x04,	// Prompt, 4 bytes
	EE_STEPPER_TABLE	=0x08,	// Stepper table, 8 bytes
	EE_HEX_TABLE		=0x10,	// Hex table, 16 bytes
	EE_PACKET_REPEAT	=0x20,	// Repeat current packet
	EE_PACKET_ACK		=0x28,	// Acknowledge a packet
	EE_PACKET_NACK		=0x30,	// Error return packet
	EE_PACKET_COMPLETE	=0x38,	// Operation complete packet
	EE_PACKET_RAIL		=0x40	// Operation interrupted packet
};

// Converts integer 0-15 to it's ASCII hex equivalent with EEPROM lookup table.
#define TOHEX(X)	EEPROM_READ((X)+EE_HEX_TABLE)

/**
 *	Different types of controllers have different chip classes.
 */
enum chipclass_e
{
	CHIP_COMM	=1,
	CHIP_STEPPER	=2,
	CHIP_EXTRUDER	=3
};

// Sends a single character, raw.  Use only for debugging.
#define SEND_CHAR(X)	do {	\
	WAIT_SERIAL();		\
	TXREG=(X);		\
	} while(0)

#endif/*__TSMPINS_H__*/
