/** 
 * tsmpic.h -- macros for programming PIC16f628a microcontrollers with SDCC.
 * Copyright (C) 2007 Tyler Montbriand
 * monttyle@burningsmell.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef __TSMPIC_H__
#define __TSMPIC_H__

#include "tsmtypes.h"

/** Turns a bit index into a bit mask.  For example,
 *  bit 0 becomes b00000001, 2 becomes b00000100, etc.
 */
#define GET_BIT(X)	(1<<(X))

/** Turns a bit index into a reverse bit mask.  For example,
 *  bit 0 becomex b11111110, 2 becomes b11111011, etc.
 */
#define MASK_BIT(X)	(~GET_BIT(X))

/** Tests if a given bit in a byte is set.  SDCC seems to be smart
 *  enough to convert these into single-bit-checking BTSS/BTSC-style 
 *  instructions.
 */
#define IS_SET(BYTE, BIT)	((BYTE) & GET_BIT(BIT))

/** Tests if a given bit in a byte is not set.  SDCC seems to be smart
 *  enough to convert these into single-bit-checking BTSS/BTSC-style 
 *  instructions.
 */
#define IS_CLEAR(BYTE,BIT)	(!IS_SET(BYTE,BIT))

/** Sets a single bit in a byte.  SDCC seems to be smart enough to
 *  turn these into single-bit-setting BSF-type instructions.
 */
#define SET_BIT(BYTE, BIT)	do { BYTE|=GET_BIT(BIT); } while(0)

/** Clears a single bit in a byte.  SDCC seems to be smart enough to
 *  turn these into single-bit-clearing BCF-type instructions.
 */
#define CLEAR_BIT(BYTE,BIT)	do { BYTE &= MASK_BIT(BIT); } while(0)

#define BIN8(Q7, Q6, Q5, Q4, Q3, Q2, Q1, Q0)	(	\
	((Q0)<<0)	|				\
	((Q1)<<1)	|				\
	((Q2)<<2)	|				\
	((Q3)<<3)	|				\
	((Q4)<<4)	|				\
	((Q5)<<5)	|				\
	((Q6)<<6)	|				\
	((Q7)<<7)	)

/** Puts the PIC into sleep mode.  This is a low power mode where it
 *  executes no instructions until waken up by some sort of interrupt.
 *  There's no C equivalent for this function so we make one with inline 
 *  assembly.
 *
 *  Note that timer interrupts will NOT wake it up!
 */
#define SLEEP() __asm SLEEP __endasm

#endif/*__TSMPIC_H__*/
